﻿define("aiassistant/editors/AIThumbnailSelector", [
    // dojo
    "dojo/_base/declare",
    "dojo/when",
    "dojo/on",
    "dojo/topic",
    "dojo/dom-class",
    "dojo/ready",
    "dojo/_base/lang",
    // dijit
    "dijit/focus",
    "dijit/ProgressBar",
    // epi
    "epi/Url",
    "epi/routes",
    "epi/shell/dnd/Target",
    "epi/shell/DialogService",
    "epi/shell/XhrWrapper",
    "epi/dependency",
    // epi-cms
    "epi-cms/core/ContentReference",
    "epi-cms/widget/MediaSelector",
    "epi-cms/widget/ThumbnailSelector",
    "epi-cms/command/UploadContent",
    "epi-cms/widget/UploadUtil",
    "epi-cms/widget/viewmodel/MultipleFileUploadViewModel",
    "epi-cms/widget/LocalFolderUploader",
    "dojo/text!./templates/AIThumbnailSelector.html",
    "dojo/text!./templates/AISelectorDialog.html",
    "aiassistant/editors/AiDialog",
    //"dijit/Dialog",
    "epi/i18n!epi/cms/nls/episerver.cms.widget.thumbnailselector"
], function (
    // dojo
    declare,
    when,
    on,
    topic,
    domClass,
    ready,
    lang,
    // dijit
    focusManager,
    ProgressBar,
    // epi
    Url,
    routes,
    Target,
    dialogService,
    XhrWrapper,
    dependency,
    // epi-cms
    ContentReference,
    MediaSelector,
    ThumbnailSelector,
    UploadContent,
    UploadUtil,
    MultipleFileUploadViewModel,
    LocalFolderUploader,
    template,
    templateDialog,
    Dialog,
    resources
) {
    var xhr = new XhrWrapper();
    return declare([ThumbnailSelector], {
        // summary:
        //      Represents the widget to select ContentReference.
        // tags:
        //      internal

        resources: resources,

        loaded: false,
        uploadDisabledMessage: "Drag and drop feature has been disabled, please use AI Assistant.",
        pluginclientid: null,
        plugindomain: null,
        pluginpath: null,
        pluginversion: null,
        pluginiconpath: null,
        plugindisableupload: false,
        pluginmodulepath: null,
        pluginWindow: null,
        activeElement: null,
        fwWin: null,
        pluginurl: null,
        timer: null,
        disableMediaSelector: null,
        listenerBound: false,
        //      List of file extensions that may be dropped onto the editor
        allowedExtensions: [],
        templateString: template,
        buildRendering: function () {
            //switching template depending on functionality
            //this.templateString = this.disableMediaSelector === "True" ? templateEpiDisabled : template;
            this.inherited(arguments);
        },
        postCreate: function () {

            ready(setTimeout(lang.hitch(this, function () {//not optimal with timeout, but else not working with _setValue when used on init
                this.loaded = true;
            }), 100));

            //this.allowedExtensions = this.pluginallowedfiletypes.replace(".", "").replace(/,/gi, "").split('.');

            this.inherited(arguments);
        },
        postFetchCR: function (data) {
            //console.log(data);

            data.contentid = dependency.resolve("epi.cms.contentEditing.command.Editing").model.contentLink;
            data.lang = dependency.resolve("epi.cms.contentEditing.command.Editing").model.languageContext.language;
            data.propname = this.get("name");
            var self = this;
            return xhr.xhrPost({
                url: self.pluginmodulepath+'imageapi/fetch',
                postData: JSON.stringify(data),
                handleAs: "text",
                headers: { "Content-Type": "application/json", "Accept": "application/json" },
                failOk: true,
                error: function (response) {
                    if (response.xhr.statusText !== "")
                        self._dialogService.alert(response.xhr.statusText);
                    else {
                        var mess = response.response.text.substring(0, response.response.text.length > 1500 ? 1500 : response.response.text.length);
                        self._dialogService.alert(mess);
                    }
                }
            });
        },
        _onAIButtonClick: function () {
        if (this.plugindisableupload!==true){
            var initSelection = this.get("value");

            if (initSelection){
                    var self = this;
                    var registry = dependency.resolve("epi.storeregistry"),//epi.shell.store.Registry
                        store = registry.get("epi.cms.contentdata"),
                        query = store.query({
                            id: initSelection,
                            //language: "en"
                        });

                    query.then(function(results) {
                        self._openDialog(window.location.origin + results.publicUrl);
                    });
                }
            else {
                this._openDialog("");
            }
        }
        else {
            this._openDialog("");
        }},
        _openDialog: function (initSelection) {

            var versionTitle = this.pluginversion;
            versionTitle = versionTitle != "Version" ? "(" + versionTitle + ")" : "";        

            this.fwWin = new Dialog({
                title: "AI Assistant Image Generator " + versionTitle,
                style: "width: 800px; height: 600px;",
                templateString: templateDialog,
                pluginpath: this.pluginpath,
                pluginversion: this.pluginversion,
                pluginmodulepath: this.pluginmodulepath,
                lang: dependency.resolve("epi.cms.contentEditing.command.Editing").model.languageContext.language,
                contentid: dependency.resolve("epi.cms.contentEditing.command.Editing").model.contentLink,
                propname: this.get("name"),
                correlationid: this.correlationid,
                ui: dojo.locale,
                initialSelection: initSelection
              });

            this.fwWin.show();

            if (!this.listenerBound) {
                this._listener = this._listener.bind(this);
                window.pluginWindow = this.fwWin;
                window.addEventListener("message", this._listener, true);
                this.listenerBound = true;
            }

            this.timer = setInterval(lang.hitch(this,
                function () {
                    if (!this.fwWin.open) {
                        window.removeEventListener("message", this._listener, true);
                        this.listenerBound = false;
                        clearInterval(this.timer);
                        this.fwWin.destroy();
                        return;
                    }
                }),
                2000);

        },
        _listener: function (event) {
            if (event.data.action == "selectedImg") {
                this.fwWin.markButton(event.data);
                return;
            }

            if (event.data.action == "unselected") {
                this.fwWin.unmarkButton();
                return;
            }

            if (event.data.action == "useImg") {
                this.fwWin.hide();
                //console.log("Selected", event.data.path);
                window.removeEventListener("message", this._listener, true);
                this.listenerBound = false;
                this.assetsChosen(event.data);
                return;
            }
        },
        assetsChosen: function (assetsChosen) {
            var self = this;
            when(self.postFetchCR(assetsChosen),
                function (data) {
                    self.focus();
                    window.focus();
                    self._setValueAndFireOnChange(data + "");
                });
        },

        _setValueAttr: function (value) {
            if (this.disableMediaSelector === "True" && this.loaded && value != null) {
                this._dialogService.alert(this.uploadDisabledMessage);
                return;
            }
            this.inherited(arguments);
        },

        _onButtonClick: function () {
            if (this.readOnly) {
                return;
            }

            if (this.disableMediaSelector === "True") {
                this._onAIButtonClick();
                return;
            }


            this.inherited(arguments);
        },

        _onClick: function () {
            if (this.readOnly) {
                return;
            }

            if (this.disableMediaSelector === "True") {
                this._onAIButtonClick();
                return;
            }


            this.inherited(arguments);
        },

        _onDrop: function () {
            if (this.readOnly) {
                event.preventDefault();
                event.stopPropagation();
                return;
            }

            if (this.disableMediaSelector === "True") {
                this._dialogService.alert(this.uploadDisabledMessage);
                event.preventDefault();
                event.stopPropagation();
                return;
            }

            this.inherited(arguments);
        }
    });
});
