﻿define("aiassistant/editors/AiTextarea", [
    "dojo/_base/declare", // declare
    "dojo/dom-style", // domStyle.set
    "dojo/dom-class",
    "dojo/on",
    "dojo/_base/lang",
    'dijit/_TemplatedMixin',
    "dijit/form/_ExpandingTextAreaMixin",
    "dijit/form/SimpleTextarea",
    "dijit/DropDownMenu",
    "dijit/MenuItem",
    "dijit/PopupMenuItem",
    "dijit/Menu",
    "dijit/form/DropDownButton",
    "dojo/text!./templates/AiTextarea.html",
    "epi/shell/XhrWrapper",
    "dojo/topic",
    "dojo/dom",
    "epi/dependency",
    "dojo/aspect",
    "dojo/text!./templates/AIPromptDialog.html",
    "aiassistant/editors/AiPromptDialog",
    "xstyle/css!./templates/AiText.css"
], function (declare,
    domStyle,
    domClass,
    on,
    lang,
    _TemplatedMixin,
    _ExpandingTextAreaMixin,
    SimpleTextarea,
    DropDownMenu,
    MenuItem,
    PopupMenuItem,
    Menu,
    DropDownButton,
    template,
    XhrWrapper,
    topic,
    dom,
    dependency,
    aspect,
    templateDialog,
    Dialog) {

    // module:
    //		dijit/form/Textarea

    var xhr = new XhrWrapper();

    return declare("aiassistant/editors/AiTextarea", [SimpleTextarea, _ExpandingTextAreaMixin, _TemplatedMixin], {
        // summary:
        //		A textarea widget that adjusts it's height according to the amount of data.
        //
        // description:
        //		A textarea that dynamically expands/contracts (changing it's height) as
        //		the user types, to display all the text without requiring a scroll bar.
        //
        //		Takes nearly all the parameters (name, value, etc.) that a vanilla textarea takes.
        //		Rows is not supported since this widget adjusts the height.
        //
        // example:
        // |	<textarea data-dojo-type="dijit/form/TextArea">...</textarea>
        oldValue: "",
        isLoading: false,
        // TODO: for 2.0, rename this to ExpandingTextArea, and rename SimpleTextarea to TextArea

        baseClass: "dijitTextBox dijitTextArea dijitExpandingTextArea AiClass",
        aiJsonResult: null,
        listenerBound: false,

        // Override SimpleTextArea.cols to default to width:100%, for backward compatibility
        cols: "",
        // Override
        templateString: template,
        winDialog: null,
        isDialogOpen: false,
        startText: "<- click the button to get AI Assistance",
        //_contextChangedHandler: null,
        postCreate: function () {
            oldValue = this.get("value");
            this.aiStatus.innerHTML = this.startText;

            var configArr = this.shortcuts.split("||")
            if (configArr[0] !== "") { //do not show dropdown if no content in drop
                var self = this;
                var menu = new DropDownMenu({ class: "aiDD" });

                // Extend the MenuItem to include a value property
                var ValueMenuItem = declare(MenuItem, {
                    _setValueAttr: function (value) {
                        this._set("value", value);
                    }
                });

                configArr.forEach(function (label) {


                    var val = label.split("|");
                    if (val.length > 1) {

                        var submenu = new Menu();

                        val.forEach(function (slabel) {
                            if (val[0] != slabel) {
                                var menuItem = new ValueMenuItem({
                                    label: slabel,
                                    value: slabel,
                                    onClick: function () {
                                        self._onAiSuggestButtonClick(null, this.value);
                                    }
                                });
                                submenu.addChild(menuItem);
                            }
                        });

                        // Add Submenu to DropDownMenu
                        menu.addChild(new PopupMenuItem({
                            label: val[0],
                            popup: submenu
                        }));

                    }
                    else {
                        var menuItem = new ValueMenuItem({
                            label: label,
                            value: label,
                            onClick: function () {
                                self._onAiSuggestButtonClick(null, this.value);
                            }
                        });

                        menu.addChild(menuItem);
                    }
                });

                // Create the Submenu
                //var submenu = new Menu();
                //submenu.addChild(new MenuItem({
                //    label: "Submenu Item"
                //}));
                //submenu.addChild(new MenuItem({
                //    label: "Another Submenu Item"
                //}));

                //// Add Submenu to DropDownMenu
                //menu.addChild(new PopupMenuItem({
                //    label: "Submenu",
                //    popup: submenu
                //}));

                var dropdownButton = new DropDownButton({
                    label: "",
                    showLabel: false,
                    dropDown: menu
                }, this.dropdownMenuButton);

                dropdownButton.startup();
                dropdownButton.clickOnDrop = 0;

                //this bellow is a work-around that the textbox get focus if you press the dropdown first, making the dropdown close.
                on(dropdownButton, "focus", function () { //this will be triggered before the textbox focus
                    dropdownButton.clickOnDrop++; //so count up
                });

                on(this, "blur", function () {//leaving the prop, reset
                    dropdownButton.closeDropDown(false);
                    dropdownButton.clickOnDrop = 0;
                });

                on(this.textbox, "focus", function () {
                    if (dropdownButton.clickOnDrop > 0) {//will only be 1 if direct hit on dropdown on focus
                        dropdownButton.closeDropDown(false);
                        dropdownButton.openDropDown();
                    }
                    dropdownButton.clickOnDrop = -100;
                });
            }

            this.inherited(arguments);
        },
        _onTextBoxBlur: function () {

        },
        _onAiActivateButtonClick: function () {
            this._onAiSuggestButtonClick();
        },
        _changeMode: function () {
            if (this.minimalmode == "True") {
                domClass.add(this.aiActivate, "hide");
                domClass.add(this.aiAssistant, "full");
            }
        },
        _onAiSuggestButtonClick: function (event, /*prompt from shortcuts*/ shortcut) {
            this._changeMode();
            var text = this.get("value").trim();
            this._doCall(text, shortcut);

            this.setAiStatus("Hold on");
            setTimeout(this.setAiStatus, 1000, "Hold on.", this, true);
            setTimeout(this.setAiStatus, 1400, "Hold on..", this, true);
            setTimeout(this.setAiStatus, 1800, "Hold on...", this, true);
            setTimeout(this.setAiStatus, 2500, "Hold on... Almost… ", this, true);
            setTimeout(this.setAiStatus, 2600, "Hold on... Almost there… ", this, true);
            setTimeout(this.setAiStatus, 3500, "Hold on... Almost there… just a", this, true);
            setTimeout(this.setAiStatus, 3700, "Hold on... Almost there… just a few ", this, true);
            setTimeout(this.setAiStatus, 3900, "Hold on... Almost there… just a few more", this, true);
            setTimeout(this.setAiStatus, 4000, "Hold on... Almost there… just a few more seconds…", this, true);
            return false;
        },
        _onAiUseSuggestionButtonClick: function () {
            this.setAiStatus(this.aiJsonResult.useStatus);
            this.oldValue = this.aiSuggestion.value;
            this._setValueAttr(this.oldValue, true);
            this.hideResultBlock();
            this._sendStats(this.aiJsonResult);
            return false;

        },
        _handleOnChange: function (newValue, priorityChange) {

            if (priorityChange && newValue != this.value && newValue != this.oldValue && !this.isLoading) {
                
                if (this.autosuggest == "True") {
                    this._doCall(newValue);
                }
            }

            this.inherited(arguments);

        },
        _doCall: function (prompt, shortcut) {
            var self = this;
            console.group("AI " + this.correlationid);
            console.log("AI Request:", shortcut, prompt);
            this.isLoading = true;
            this.setAiSpinner(true);
            //this.hideResultBlock();
            xhr.xhrPost({
                url: this.pluginmodulepath + 'api/suggest',
                handles: 'json',
                postData: dojo.toJson({ correlationid: this.correlationid, text: prompt, lang: dependency.resolve("epi.cms.contentEditing.command.Editing").model.languageContext.language, contentid: dependency.resolve("epi.cms.contentEditing.command.Editing").model.contentLink, propname: this.get("name"), shortcut: shortcut, ui: dojo.locale }),
                headers: { "Content-Type": "application/json", "Accept": "application/json" },
                load: function (jsonText) {
                    self.isLoading = false;//we got result
                    self.setAiSpinner(false);
                    self.aiJsonResult = JSON.parse(jsonText);
                    console.log("AI Response:", self.aiJsonResult.shortcut, self.aiJsonResult.text);
                    console.groupEnd();
                    if (self.aiJsonResult.runCommand) {
                        self.isLoading = true;
                        self.setAiStatus(self.startText);
                        self.runCommand(self.aiJsonResult.runCommand, self.aiJsonResult.text);
                        return;
                    }
                    
                    self.setAiStatus(self.aiJsonResult.aiStatus);//get this from result
                    self.setUseButton(self.aiJsonResult.useButton);
                    self.setAiLicenseHtml(self.aiJsonResult.licenseHtml);
                    self.setAiLimitationHtml(self.aiJsonResult.limitations);
                    if (self.aiJsonResult.openResult) {
                        self.setAiSuggestion(self.aiJsonResult.text);
                        self.showResultBlock();
                    }
                    else {
                        self.hideResultBlock();
                    }
                }
            });
        },
        runCommand: function (command) {
            if(command == "prompt") {
                this._openPromptDialog();
            }
        },
        _openPromptDialog: function () {

            this.isDialogOpen = true;
            var versionTitle = this.pluginversion;
            versionTitle = versionTitle != "Version" ? "(" + versionTitle + ")" : "";

            this.winDialog = new Dialog({
                title: "Ask AI Assistant " + versionTitle,
                style: "width: 800px; height: 600px;",
                templateString: templateDialog,
                pluginpath: this.pluginpath,
                pluginversion: this.pluginversion,
                pluginmodulepath: this.pluginmodulepath,
                propname: this.get("name"),
                correlationid: this.correlationid,
                lang: dependency.resolve("epi.cms.contentEditing.command.Editing").model.languageContext.language,
                contentid: dependency.resolve("epi.cms.contentEditing.command.Editing").model.contentLink,
                ui: dojo.locale,
                isHtml: false,
                defaultText: "",
                initialSelection: ""
            });

            this.winDialog.show();

            if (!this.listenerBound) {
                this._listener = this._listener.bind(this);
                window.pluginWindow = this.winDialog;
                window.addEventListener("message", this._listener, true);
                this.listenerBound = true;
            }

            this.timer = setInterval(lang.hitch(this,
                function () {
                    if (!this.winDialog.open) {
                        window.removeEventListener("message", this._listener, true);
                        this.listenerBound = false;
                        this.isDialogOpen = false;
                        clearInterval(this.timer);
                        this.winDialog.destroy();
                        return;
                    }
                }),
                2000);

        },
        _listener: function (event) {
            if (event.data.action == "getContent" && this.value) {

                var val = this.textbox;  
                var iframe = document.getElementById("aiPromptFrame");
                if (iframe) {
                    iframe.contentWindow.postMessage({ "action": "setContent", "content": this.value }, location.origin);
                }
                return;
            }
            if (event.data.action == "selected") {
                this.winDialog.markButton(event.data.data);
                this.isDialogOpen = true;
                return;
            }

            if (event.data.action == "unselected") {
                this.winDialog.unmarkButton();
                return;
            }

            if (event.data.action == "use") {
                this.isLoading = false;
                window.removeEventListener('message', this._listener);
                this.winDialog.hide();
                this.isDialogOpen = false;
                this.focus();
                window.focus();
                this.aiJsonResult = event.data.data;
                this.aiSuggestion.value = this.aiJsonResult.text;//set the text
                this._onAiUseSuggestionButtonClick();//use same logic as button click

                return;
            }
        },
        _sendStats: function (json) {
            xhr.xhrPost({
                url: this.pluginmodulepath +'api/stats',
                handles: 'json',
                postData: dojo.toJson(json),
                headers: { "Content-Type": "application/json", "Accept": "application/json" },
                load: function (jsonText) {
                   // JSON.parse(jsonText);
                }
            });
        },
        setAiStatus(value, plugin, whileloading) {

            if (whileloading && plugin)  {//only if loading
                if (plugin.isLoading && !plugin.isDialogOpen)//and is still loading and not dialog open
                    plugin.aiStatus.innerHTML = value;//then show this
            }
            else
                this.aiStatus.innerHTML = value;
        },
        setAiSpinner(spin) {
            dojo.toggleClass(this.aiSpinner, "spin", spin);
        },
        setAiSuggestion(value) {
            this.aiSuggestion.value = value;
        },
        setUseButton(value) {
            this.aibutton.innerHTML = value;
        },
        setAiLicenseHtml(value) {
            if (value == "" || value == null || value == undefined) {
                this.aiLicense.style.display = "none";
            } else {
                this.aiLicense.style.display = "";
                this.aiLicense.innerHTML = value;
            }
        },
        setAiLimitationHtml(value) {
            if (value == "" || value == null || value == undefined) {
                this.aiLimitation.style.display = "none";
            } else {
                this.aiLimitation.style.display = "";
                this.aiLimitation.innerHTML = value;
            }
        },
        hideResultBlock() {
            this.aiDivResult.style.display = "none";
        },

        showResultBlock() {
            this.aiDivResult.style.display = "block";
        },
        isNullOrEmpty(str) {
        return str === null || str === undefined || String(str).trim() === '';
        },
        buildRendering: function () {
            this.inherited(arguments);

            this.baseClass = this.baseClass + " type" + this.inputtype
            
            if (this.minimalmode == "True") {
                domClass.add(this.aiAssistant, "minimal");
                domClass.remove(this.aiActivate, "hide");
            }

            if (this.inputtype == "input") { //act as inputfield, no line breaks
                this.textbox.addEventListener("keyup", function (event) {
                    const keyCode = event.keyCode || event.which;

                    // Remove line breaks on Enter key (keyCode 13)
                    if (keyCode === 13) {
                        event.preventDefault();
                        this.value = this.value.replace(/(\r\n|\n|\r)/gm, "");
                    }
                })
                domStyle.set(this.textbox, { whiteSpace: 'nowrap', overflowY: 'hidden', overflowX: 'auto', boxSizing: 'border-box', MsBoxSizing: 'border-box', WebkitBoxSizing: 'border-box', MozBoxSizing: 'border-box' });
            }
            else
            // tweak textarea style to reduce browser differences
            domStyle.set(this.textbox, { overflowY: 'hidden', overflowX: 'auto', boxSizing: 'border-box', MsBoxSizing: 'border-box', WebkitBoxSizing: 'border-box', MozBoxSizing: 'border-box' });
        }
    });

});