﻿"use strict";

var _searchcounter = 0;

var initpagescript = async function () {


    const iframe = document.getElementById('mainframe');

    const selectUrl = getQueryStringParameterByName('selectUrl');
    const version = getQueryStringParameterByName('version');
    const defaultText = getQueryStringParameterByName('defaultText');
    const propname = getQueryStringParameterByName('propname');
    const contentid = getQueryStringParameterByName('contentid');
    const lang = getQueryStringParameterByName('lang');
    const correlationid = getQueryStringParameterByName("correlationid");
    const pluginmodulepath = getQueryStringParameterByName("pluginmodulepath");
    const ui = getQueryStringParameterByName("ui");
    window.__ui = ui;
    //make it global
    window.pageData = {
        "correlationid": correlationid,
        "propname": propname,
        "lang": lang,
        "contentid": contentid
    };


    // init messaging
    window.addEventListener('message', messageEvent);

    /*
     * Handle message events.
     *
     * Forwards from opener to Iframe or returns from Iframe to opener.
     */
    function messageEvent(message) {
        if (verifyInputMessage(message)) {
            // forward message -> to Iframe
            if (iframe)
                iframe.contentWindow.postMessage(message.data, ivOrigin);
        } else {
            // reply to opener <- from Iframe
            if (window.opener)
                window.opener.postMessage(message.data, location.origin);
            else
                window.parent.postMessage(message.data, location.origin);
        }
    }


    /*
     * Verifies a message as input to Iframe:
     * Check that message origin is from us and make sure the message data is a valid input command.
     */
    function verifyInputMessage(message) {
        return message.origin === location.origin;
    }

    /*
     * Helper method to get specific query parameter from querystring
     */
    function getQueryStringParameterByName(name) {
        const url = window.location.href;
        name = name.replace(/[\[\]]/g, '\\$&');
        const regex = new RegExp('[?&]' + name + '(=([^&#]*)|&|#|$)');
        const results = regex.exec(url);

        if (!results) {
            return '';
        }

        if (!results[2]) {
            return '';
        }

        return decodeURIComponent(results[2].replace(/\+/g, ' '));
    }

    function initFunction() {
        const url = pluginmodulepath + "imageapi/init";
        var searchEl = document.getElementById('imgAiImageForm_txtSearch');
      
        hideImageLists();
        showOverlay();
        const data = {
            "text": searchEl.value,
            "correlationid": correlationid,
            "propname": propname,
            "lang": lang,
            "ui": ui,
            "contentid": contentid
        };

        fetch(url, {
            method: "POST",
            headers: {
                "Content-Type": "application/json"
            },
            body: JSON.stringify(data)
        })
            .then((response) => {

                hideOverlay();

                response.json().then(data => {
                    var div = document.getElementById('aiImageResponse');
                    if (!isNullOrEmpty(data.licenseHtml)) {
                        var pElement = document.createElement('p');
                        pElement.className = "red";
                        pElement.innerHTML = data.licenseHtml;
                        div.prepend(pElement);
                    }
                    if (!isNullOrEmpty(data.limitations)) {
                        var pElement = document.createElement('p');
                        pElement.innerHTML = data.limitations;
                        div.prepend(pElement);
                    }
                    if (data.enableGenerate) {
                        document.getElementById('btnGenerate').disabled = data.EnableGenerate;
                        document.getElementById('imgAiImageForm_txtSearch').disabled = data.EnableGenerate;
                        showImageLists();
                        if (defaultText !== "") {
                            document.getElementById('imgAiImageForm_txtSearch').value = defaultText;
                            //_suggestImages(); not automatic anymore
                        }
                    }                    
                });

            })
            .catch((error) => {
                console.error("Error:", error);
            });
    }
    function _suggestImages() {
        _generateImageFromText(document.getElementById('imgAiImageForm_txtSearch').value);
        return false;
    }

    function _generateImageFromText(prompt) {
        const url = pluginmodulepath + "imageapi/generate";
        const data = {
            "text": prompt,
            "correlationid": correlationid,
            "propname": propname,
            "lang": lang,
            "ui": ui,
            "contentid": contentid
        };

        showOverlay();

        fetch(url, {
            method: "POST",
            headers: {
                "Content-Type": "application/json"
            },
            body: JSON.stringify(data)
        })
            .then((response) => {

                _searchcounter++;//this make the button history show up
                _unselectAsset();
                hideImageLists();
                hideOverlay();

                if (!response.ok) {
                    response.json().then(data => {
                        writeError(data);
                    });
                    return;
                }

                _searchcounter++;

                if (defaultText !== "" && selectUrl !== "" && _searchcounter == 0) {//this we have probably opened from TinyMce and had an image selected
                    showImageLists();
                }


                var div = document.getElementById('aiImageResponse');
                const ulElement = document.createElement('ul');8
                response.json().then(data => {
                    ulElement.className = "imagelist cols" + data.images.length;
                    for (var i = 0; i <= data.images.length - 1; i++) {
                        const liElement = document.createElement('li');
                        const imgEl = document.createElement('img');
                        imgEl.src = data.images[i].url;
                        imgEl.className = "selectable";
                        imgEl.dataset.userinput = prompt;
                        imgEl.dataset.prompt = data.images[i].revised_prompt;
                        liElement.appendChild(imgEl);
                        ulElement.appendChild(liElement);
                    }
                    div.appendChild(ulElement);
                });

            })
            .catch((error) => {
                console.error("Error:", error);
                document.getElementById('errormessage').innerHTML = error;
            });
        return false;
    }


    try {

        window._suggestImages = _suggestImages;
        window._generateImageFromText = _generateImageFromText;
        window._generateVariation = _generateVariation;

        if (!isNullOrEmpty(selectUrl)) {

            var div = document.getElementById('aiImageResponse');
            const ulElement = document.createElement('ul');
            ulElement.className = "imagelist original";
            const liElement = document.createElement('li');
            const imgEl = document.createElement('img');
            imgEl.src = selectUrl;
            imgEl.className = "selectable noUse";
            liElement.appendChild(imgEl);
            ulElement.appendChild(liElement);
            div.appendChild(ulElement);
        }

        initFunction();
        //here do init stuff
        console.log("AI Assistant version =>", version);

        // Attach the event to the body element
        document.body.addEventListener('click', function (event) {
            // Check if the clicked element is an image
            //console.log('body clicked!', event.target.tagName, event.target.className);

            if (event.target.tagName === 'IMG' && event.target.classList.contains("selectable")) {

                if (event.target.classList.contains('selected')) {
                    _unselectAsset();
                }
                else {
                    _unselectAsset();
                    event.target.classList.toggle('selected');
                    if (!event.target.classList.contains("noUse"))//do not use this one
                    {
                        _selectAsset(event.target.src, event.target.dataset.prompt);
                    }
                    //add buttons for variations
                    var templateElement = document.getElementById('buttonTemplate');
                    var container = document.createElement('div');
                    container.className = "actions";
                    container.innerHTML = templateElement.innerHTML;
                    event.target.parentNode.appendChild(container);
                }     
            }
        });



    } catch (error) {

        // Log error
        console.error(error);
        document.getElementById('errormessage').innerHTML = error;
    }
}

function _selectAsset(path, name) {
    var data = { "action": "selectedImg", "path": path, "name": name };
    window.parent.postMessage(data, location.origin);
}

function _unselectAsset() {

    var firstSelectedImg = document.querySelector('img.selected');
    // Check if the element exists
    if (firstSelectedImg) {
        // Perform actions with the first selected image
        //console.log("Unselected", firstSelectedImg.src);
        firstSelectedImg.classList.toggle('selected');
    } 

    // Find elements using a CSS selector
    var elementsToRemove = document.querySelectorAll('.actions');

    // Remove each element from its parent
    elementsToRemove.forEach(function (element) {
        element.parentNode.removeChild(element);
    });

    var data = { "action": "unselected", "path": "" };
    if (window.opener)
        window.opener.postMessage(data, location.origin);
    else
        window.parent.postMessage(data, location.origin);
}

function hideImageLists() {

    document.getElementById('errormessage').innerHTML = "";
    // Get all ul elements with the class "imagelist"
    var ulList = document.querySelectorAll('ul.imagelist');

    // Iterate through each ul element
    ulList.forEach(function (ulElement) {
        // Toggle the class "hidden" on the ul element
        if (!ulElement.classList.contains('hidden'))
            ulElement.classList.toggle('hidden');
    });
    if (_searchcounter > 0) {
        var btnHistory = document.getElementById('btnHistory');
        if (btnHistory)
            btnHistory.classList.remove("hidden");
    }
}

function showImageLists() {
    // Get all ul elements with the class "imagelist"
    var ulList = document.querySelectorAll('ul.imagelist');

    // Iterate through each ul element
    ulList.forEach(function (ulElement) {
        // Toggle the class "hidden" on the ul element
        if (ulElement.classList.contains('hidden')) {
            ulElement.classList.toggle('hidden');
        }
    });
}

function _enlarge(el) {
    showImageOverlay(el.parentNode.parentNode.firstElementChild.src);
}

function showImageOverlay(imageUrl) {
    var overlay = document.getElementById('overlay2');
    var overlayImage = document.getElementById('overlayImage');
    overlayImage.src = imageUrl;
    overlayImage.addEventListener('click', hideImageOverlay);
    overlay.addEventListener('click', hideImageOverlay);
    overlay.style.display = 'flex';
}

function hideImageOverlay() {
    var overlay = document.getElementById('overlay2');
    overlay.style.display = 'none';
}


function _generateFromText(element) {
    // Get the parent <li> element
    var parentLi = element.closest('li');

    //hide every
    parentLi.parentNode.querySelectorAll('.variation-form')?.forEach(function (el) {
        el.setAttribute("class", "hidden");
    });

    // Check if the form has already been generated by looking for a specific class or identifier
    if (!parentLi.parentNode.querySelector('.variation-form.fromtext')) {
        var img = parentLi.querySelector('img.selectable');
        // Get the data-prompt attribute value from the image
        var promptText = img.getAttribute('data-prompt');

        // Create a new <li> element
        var newLi = document.createElement('li');
        newLi.classList.add('variation-form');

        // Create a <textarea> and set its value to the prompt text
        var textarea = document.createElement('textarea');
        textarea.value = promptText;
        textarea.rows = 4; // Set the desired rows
        textarea.cols = 50; // Set the desired cols
        textarea.placeholder = "Add anything to the image? ";
        // Append the <textarea> and buttons to the new <li>
        newLi.appendChild(textarea);

        // Create the "Generate from text" button
        var genFromTextBtn = document.createElement('button');
        genFromTextBtn.textContent = 'Generate from text';
        genFromTextBtn.onclick = function () {
            _generateImageFromText(textarea.value);
        };

        newLi.appendChild(genFromTextBtn);

        if (isNullOrEmpty(promptText)) {
            _getDescription(img, textarea);
        }

        // Append the new <li> to the <ul>
        parentLi.parentNode.appendChild(newLi);
        adjustTextareaHeight(textarea);
        genFromTextBtn.scrollIntoView({ behavior: 'smooth', block: 'end' });
    }
    else {
        parentLi.parentNode.querySelector('.variation-form.fromtext').classList.remove("hidden");
    }
}

function _generateVariation(element) {
    // Get the parent <li> element
    var parentLi = element.closest('li');

    //hide every
    parentLi.parentNode.querySelectorAll('.variation-form')?.forEach(function (el) {
        el.setAttribute("class", "hidden");
    });
    // Check if the variation has already been generated by looking for a specific class or identifier
    if (!parentLi.parentNode.querySelector('.variation-form.variation')) {
        var img = parentLi.querySelector('img.selectable');
        // Get the data-prompt attribute value from the image
        //var promptText = img.getAttribute('data-prompt');

        // Create a new <li> element
        var newLi = document.createElement('li');
        newLi.classList.add('variation-form');
        newLi.classList.add('variation');

        // Create a <textarea> and set its value to the prompt text
        var textarea = document.createElement('textarea');
        //textarea.value = promptText;
        textarea.rows = 4; // Set the desired rows
        textarea.cols = 50; // Set the desired cols
        textarea.placeholder = "Add anything to the image? ";
        // Append the <textarea> and buttons to the new <li>
        newLi.appendChild(textarea);
        
        // Create the "Generate with addings" button
        var genWithAddingsBtn = document.createElement('button');
        genWithAddingsBtn.textContent = 'Regenerate image';
        genWithAddingsBtn.onclick = function () {
            _onImageVariation(img.src, textarea.value);
        };

        newLi.appendChild(genWithAddingsBtn);

        // Append the new <li> to the <ul>
        parentLi.parentNode.appendChild(newLi);
        adjustTextareaHeight(textarea);
        genWithAddingsBtn.scrollIntoView({ behavior: 'smooth', block: 'end' });
    }
    else {
        parentLi.parentNode.querySelector('.variation-form.variation').classList.remove("hidden");
    }
}

function adjustTextareaHeight(textarea) {
    // Reset height to auto to ensure scrollHeight gets the correct value
    textarea.style.height = 'auto';
    // Set the height to the scroll height to fit the content
    textarea.style.height = textarea.scrollHeight + 'px';
}

function _getDescription(image, textarea) {

    const data = {
        "text": image.src,
        "correlationid": window.pageData.correlationid,
        "propname": window.pageData.propname,
        "lang": window.pageData.lang,
        "contentid": window.pageData.contentid,
        "ui": window.__ui,
        "prompt": "[IMGDESC]"
    };

    showOverlay();
    fetch("../../../imageapi/variations", {
        method: "POST",
        headers: {
            "Content-Type": "application/json"
        },
        body: JSON.stringify(data)
    }).then((response) => {

        hideOverlay();

        if (!response.ok) {
            response.json().then(data => {
                writeError(data);
            });
            return;
        }

        response.json().then(data => {
            textarea.value = data.text;
            image.dataset.prompt = data.text;
        });

    })
        .catch((error) => {
            console.error("Error:", error);
            document.getElementById('errormessage').innerHTML = error;
        });

}

function _onImageVariation(imagePath, prompt) {
    
    const data = {
        "text": imagePath,
        "correlationid": window.pageData.correlationid,
        "propname": window.pageData.propname,
        "lang": window.pageData.lang,
        "contentid": window.pageData.contentid,
        "ui": window.__ui,
        "prompt": prompt
    };

    showOverlay();
    fetch("../../../imageapi/variations", {
            method: "POST",
            headers: {
                "Content-Type": "application/json"
            },
            body: JSON.stringify(data)
        }).then((response) => {

            _searchcounter++;

            _unselectAsset();
            hideImageLists();
            hideOverlay();

            if (!response.ok) {
                response.json().then(data => {
                    writeError(data);
                });
                return;
            }

            response.json().then(data => {
                var div = document.getElementById('aiImageResponse');
                const ulElement = document.createElement('ul');
                ulElement.className = "imagelist cols" + data.images.length;
                var txtInput = document.getElementById('imgAiImageForm_txtSearch');
                for (var i = 0; i <= data.images.length - 1; i++) {
                    const liElement = document.createElement('li');
                    const imgEl = document.createElement('img');
                    imgEl.className = "selectable";
                    imgEl.dataset.userinput = prompt;
                    imgEl.dataset.prompt = data.images[i].revised_prompt;
                    imgEl.src = data.images[i].url;
                    liElement.appendChild(imgEl);
                    ulElement.appendChild(liElement);
                }
                div.appendChild(ulElement);
            });

        })
        .catch((error) => {
            console.error("Error:", error);
            document.getElementById('errormessage').innerHTML = error;
        });
};

function writeError(data) {

        if (data.Message) {
            document.getElementById('errormessage').innerHTML = data.Message;
            console.error(data);
            return;
        }
        if (data.ActualMessage) {
            document.getElementById('errormessage').innerHTML = data.ActualMessage;
            console.error(data);
            return;
        }
        if (data.InnerException && data.InnerException.Message) {
            document.getElementById('errormessage').innerHTML = data.ActualMessage + data.InnerException.Message;
            console.error(data);
            return;
        }
        document.getElementById('errormessage').innerHTML = data.error.message;
        console.error(data)

}


document.addEventListener('DOMContentLoaded', function () { initpagescript("") });

function showOverlay() {
    document.getElementById('overlay').style.display = 'flex';
}

function hideOverlay() {
    document.getElementById('overlay').style.display = 'none';
}

function isNullOrEmpty(str) {
    return str === null || str === undefined || String(str).trim() === '';
}