﻿"use strict";

var _searchcounter = 0;

var initpagescript = async function () {   

    const selectUrl = getQueryStringParameterByName('selectUrl');
    const version = getQueryStringParameterByName('version');
    const defaultText = getQueryStringParameterByName('defaultText');
    const propname = getQueryStringParameterByName('propname');
    const contentid = getQueryStringParameterByName('contentid');
    const lang = getQueryStringParameterByName('lang');
    const isHtml = getQueryStringParameterByName('isHtml');
    const correlationid = getQueryStringParameterByName("correlationid");
    const pluginmodulepath = getQueryStringParameterByName("pluginmodulepath");
    const ui = getQueryStringParameterByName("ui");

    // init messaging
    window.addEventListener('message', messageEvent);

    /*
     * Handle message events.
     *
     * Forwards from opener to Iframe or returns from Iframe to opener.
     */
    function messageEvent(message) {
        
        if (message.data.action == "setContent") {
            if (isHtml == "true") {
                document.getElementById('aiRequest').innerHTML = "" + message.data.content;
            }
            else
            {
                document.getElementById('aiRequest').classList.add("wrap");
                document.getElementById('aiRequest').innerHTML = "" + message.data.content;
            }
            return;
        }


        if (verifyInputMessage(message)) {
            //incoming message from opener -> to Iframe
        } else {
            // reply to opener <- from Iframe
            window.parent.postMessage(message.data, location.origin);
        }
    }


    /*
     * Verifies a message as input to Iframe:
     * Check that message origin is from us and make sure the message data is a valid input command.
     */
    function verifyInputMessage(message) {
        return message.origin === location.origin;
    }

    /*
     * Helper method to get specific query parameter from querystring
     */
    function getQueryStringParameterByName(name) {
        const url = window.location.href;
        name = name.replace(/[\[\]]/g, '\\$&');
        const regex = new RegExp('[?&]' + name + '(=([^&#]*)|&|#|$)');
        const results = regex.exec(url);

        if (!results) {
            return '';
        }

        if (!results[2]) {
            return '';
        }

        return decodeURIComponent(results[2].replace(/\+/g, ' '));
    }
    
    function _suggest() {
        var prompt = document.getElementById('aiForm_txtSearch').value;
        var text = document.getElementById('aiRequest').innerHTML;

        if (isNullOrEmpty(prompt) && isNullOrEmpty(text))
            return;

        const data = {
            "text": text,
            "prompt": prompt,
            "correlationid": correlationid,
            "propname": propname,
            "lang": lang,
            "isHtml": isHtml == "true",
            "contentid": contentid,
            "ui": ui
        };

        console.group("AI " + correlationid);

        console.log("AI Request:", prompt);

        showOverlay();

        fetch(pluginmodulepath + "api/suggest", {
            method: "POST",
            headers: {
                "Content-Type": "application/json"
            },
            body: JSON.stringify(data)
        })
            .then((response) => {

                _unselectAsset();
                hideOverlay();

                if (!response.ok) {
                    response.json().then(data => {
                        if (data.Message) {
                            document.getElementById('errormessage').innerHTML = data.Message;
                            console.error(data);
                            return;
                        }
                        if (data.ActualMessage) {
                            document.getElementById('errormessage').innerHTML = data.ActualMessage;
                            console.error(data);
                            return;
                        }
                        if (data.InnerException && data.InnerException.Message) {
                            document.getElementById('errormessage').innerHTML = data.ActualMessage + data.InnerException.Message;
                            console.error(data);
                            return;
                        }
                        document.getElementById('errormessage').innerHTML = data.error.message;
                        console.error(data)
                    });
                    console.groupEnd();
                    return;
                }                

                response.json().then(data => {

                    if (!data.openResult) {
                        document.getElementById('errormessage').innerHTML = data.aiStatus;
                        console.error(data);
                        return;
                    }
                    document.getElementById('errormessage').innerHTML = "";

                    console.log("AI Response:", data.text);
                    console.groupEnd();
                    // clean up old message, select all elements with the class 'red'
                    var elements = document.querySelectorAll('#messages p');

                    // Loop through the NodeList and remove each element
                    elements.forEach(function (element) {
                        element.parentNode.removeChild(element);
                    });


                    var div = document.getElementById('messages');

                    if (data.generateImage) {
                        var pElement = document.createElement('p');
                        pElement.className = "red";
                        pElement.innerHTML = data.aiStatus;
                        div.appendChild(pElement);
                    }
                    else {
                        //use formatText if isHtml is false
                        if (isHtml == "true") {
                            data.text = formatText(data.text);
                        }
                        else if (/\n/.test(data.text)) {//if br
                            //document.getElementById('aiResponse1').classList.add("wrap");
                            data.text = formatTextToHtml(data.text);
                         }

                        addResponseTab(data);
                    }

                    if (!isNullOrEmpty(data.limitations)) {
                        var pElement = document.createElement('p');
                        pElement.innerHTML = data.limitations;
                        div.appendChild(pElement);
                    }

                    if (!isNullOrEmpty(data.licenseHtml)) {
                        var pElement = document.createElement('p');
                        pElement.className = "red";
                        pElement.innerHTML = data.licenseHtml;
                        div.appendChild(pElement);
                    }
                });

            })
            .catch((error) => {
                console.error("Error:", error);
                console.groupEnd();
                document.getElementById('errormessage').innerHTML = error;
            });
        return false;
    }

    var addResponseTab = function (data) {
        // Increment the count each time the button is clicked
        responseTabCount++;

        if (responseTabCount === 1) {
            var tabContent = document.getElementById('aiResponse1');
            tabContent.innerHTML = data.text;

            var btn = document.getElementById('responseTab1');
            btn.aidata = data;
            tabContent.btnTab = btn;
            btn.click();
            tabContent.addEventListener('keyup', function () {
                addTextToModel(this);
            });
            return;
        }

        // Create the new tab button
        var newTabButton = document.createElement('button');
        newTabButton.className = 'tabButton';
        newTabButton.dataset.count = responseTabCount;
        newTabButton.aidata = data;
        newTabButton.id = 'responseTab' + responseTabCount;
        newTabButton.textContent = 'Response ' + responseTabCount;
        // Adjust the onclick function to target the correct content
        newTabButton.setAttribute('onclick', "openTab(event, 'aiResponse" + responseTabCount + "')");

        // Create the new tab content
        var newTabContent = document.createElement('div');
        newTabContent.id = 'aiResponse' + responseTabCount;
        newTabContent.className = 'tabContent';
        newTabContent.btnTab = newTabButton;
        newTabContent.setAttribute("contenteditable", true);
        newTabContent.addEventListener('keyup', function () {
            addTextToModel(this);
        });
        newTabContent.style.display = 'none';
        newTabContent.innerHTML = data.text; // Placeholder text

        // Append the new elements to the DOM
        document.getElementById('tabContainer').appendChild(newTabButton);
        document.body.appendChild(newTabContent); // Append new content div to body or to a specific container as per your layout

        newTabButton.click();
    } 

    try {

        window._suggest = _suggest;

        //here do init stuff

        document.getElementById('aiRequest').innerHTML = "";
        document.getElementById('aiRequest').contentEditable = true;
        
        document.getElementById('requestTab').click();

        var data = { "action": "getContent" };
        if (window.opener)
            window.opener.postMessage(data, location.origin);
        else
            window.parent.postMessage(data, location.origin);

        setTimeout(document.getElementById('aiForm_txtSearch').focus(), 1000);


        var responseTabCount = 0;

        //only for debug
        //uncomment the button in plugin.html
        if (document.getElementById('addResponse'))
            document.getElementById('addResponse').addEventListener('click', function () { addResponseTab("some input " + responseTabCount) });

    } catch (error) {

        // Log error
        console.error(error);
        document.getElementById('errormessage').innerHTML = error;
    }
}

function addTextToModel(self) {
    self.btnTab.aidata.text = self.innerHTML;
    console.log("isHtml", self.isHtml)
    if (self.isHtml !== "true") {
        self.btnTab.aidata.text = formatHtmlToText(self.innerHTML)
    }
    _selectAsset(self.btnTab.aidata);
    //console.log('Content changed!', this.innerHTML, this.btnTab.aidata.text);
}

function formatText(text) {
    // Check if the text contains HTML tags
    if (/<\/?[a-z][\s\S]*>/i.test(text)) {
        return text; // Return the original text if HTML is found
    }

    // Check if the text contains newline characters
    if (/\n/.test(text)) {
        // Replace newline characters with <br> tags
        return text.replace(/\n/g, '<br>');
    }

    return text;
}

function formatTextToHtml(text) {
    
    // Check if the text contains newline characters
    if (/\n/.test(text)) {
        // Replace newline characters with <br> tags
        text = text.replace(/\n/g, '<br>');
        console.log("Replace text to br tags", text )
    }

    return text;
}

function formatHtmlToText(text) {
    // Check if the text contains HTML tags
    if (/<\/?[a-z][\s\S]*>/i.test(text)) {
        text = text.replace(/<br>/g, '\n');
        text = text.replace(/<[^>]*>/g, '');

        console.log("Replace html to text", text)
    }

    return text;
}

function _selectAsset(json) {
    var data = { "action": "selected", "data": json };
    window.parent.postMessage(data, location.origin);
}

function openTab(evt, tabName) {
    // Get all elements with class="tabContent" and hide them
    var tabcontent = document.getElementsByClassName("tabContent");
    for (var i = 0; i < tabcontent.length; i++) {
        tabcontent[i].style.display = "none";
    }

    // Get all elements with class="tabButton" and remove the class "active"
    var tablinks = document.getElementsByClassName("tabButton");
    for (i = 0; i < tablinks.length; i++) {
        tablinks[i].className = tablinks[i].className.replace(" active", "");
    }

    // Show the current tab, and add an "active" class to the button that opened the tab
    document.getElementById(tabName).style.display = "block";
    evt.currentTarget.className += " active";
    if (evt.currentTarget.aidata) {
        _selectAsset(evt.currentTarget.aidata);   
    }
}

function switcher(evt) {
    // Get all elements with class="tabContent" and hide them
    var tab = document.getElementsByClassName("active");
    if (tab.length > 0) {
        var count = tab[0].dataset.count;//input tab do not has any count
        if (count) {
            var text = document.getElementById("aiResponse" + count).innerHTML;
            if (text.trim() != "") {
                document.getElementById('aiRequest').innerHTML = text;
                document.getElementById('requestTab').click();
            }
        }
    }
}

function _unselectAsset() {
    window.parent.postMessage({ "action": "unselected", "path": "" }, location.origin);
}


document.addEventListener('DOMContentLoaded', function () { initpagescript("") });

function showOverlay() {
    document.getElementById('overlay').style.display = 'flex';
}

function hideOverlay() {
    document.getElementById('overlay').style.display = 'none';
}

function isNullOrEmpty(str) {
    return str === null || str === undefined || String(str).trim() === '';
}